// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "GetMultiDataRefRequest.h"

#include "Protocol.h"

#include <hu/varadiistvan/xplcommon/Util.h>

//------------------------------------------------------------------------------

using xplra::GetMultiDataRefRequest;

using hu::varadiistvan::scpl::io::DataStream;
using hu::varadiistvan::xplcommon::Util;

//------------------------------------------------------------------------------

GetMultiDataRefRequest::GetMultiDataRefRequest(uint8_t& result,
                                               DataStream& stream)
{
    result = Protocol::RESULT_OK;

    uint32_t numTasks = stream.readU32();
    if (!stream) return;
    if (numTasks==0 || numTasks>Protocol::MAX_MULTI_COUNT) {
        result = Protocol::RESULT_INVALID_COUNT;
        return;
    }

    for(size_t i = 0;
        i<numTasks && result==Protocol::RESULT_OK && stream; ++i)
    {
        GetDataRefTask* task = GetDataRefTask::create(result, stream);
        if (task!=0) addTask(task);
    }
}

//------------------------------------------------------------------------------

void GetMultiDataRefRequest::writeResult(DataStream& stream) const
{
    // Util::debug("hu.varadiistvan.xplra.GetMultiDataRefRequest[%p]::writeResult\n", this);
    size_t numTasks = getNumTasks();
    for(size_t i = 0; i<numTasks; ++i) {
        GetDataRefTask* task = getTask(i);
        // Util::debug("hu.varadiistvan.xplra.GetMultiDataRefRequest[%p]::writeResult: task[%zu]: %d @ %p\n",
        //             this, i, task->isValid(), task);
        if (!task->isValid()) {
            stream.writeU8(Protocol::RESULT_UNKNOWN_DATAREF);
            stream.writeU32(i);
            return;
        }
    }

    stream.writeU8(Protocol::RESULT_OK);
    for(size_t i = 0; i<numTasks; ++i) {
        GetDataRefTask* task = getTask(i);
        task->writeValue(stream);
    }
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
